#include <malloc.h>
#include <string.h>
#include <math.h>


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#ifdef _WIN32
#include "ext/standard/info.h"
#endif

#include "greekutils.h"
#include "color.h"

#ifndef _WIN32
	#include "color.c"
#endif

#define MAX_LEN		1024	//max string length. 512 bytes should be enough for everyone! Do you need more ?

enum tagLanguages {
	el = 0,
	en,
	fr,
	it,
	de,
	es,
	po,
	tagLanguagesCount
};

char* languageNames[] = {
	"Ελληνικά",
	"English",
	"Française",
	"Italiano",
	"Deutsche",
	"Español",
	"Polski",
	NULL
};

static enum tagLanguages _defaultLanguage = el;

enum tagLanguages getDefaultLanguage()
{
	return _defaultLanguage;
}

void setDefaultLanguage(enum tagLanguages lang)
{
	_defaultLanguage = lang;
}

char* getDefaultLanguageName()
{
	return languageNames[_defaultLanguage];
}

char* getLanguageName(enum tagLanguages lang)
{
	return languageNames[lang];
}


//-----------------------------------------------------------------------------

char* dayNames[][7] = {
	{"Κυριακή",		"Δευτέρα",		"Τρίτη",	"Τετάρτη",		"Πέμπτη",		"Παρασκευή",	"Σαββάτο"},		//el
	{"Sunday",		"Monday",		"Tuesday",	"Wednesday",	"Thursday",		"Friday",		"Saturday"},	//en
	{"Dimanche",	"Lundi",		"Mardi",	"Mercredi",		"Jeudi",		"Vendredi",		"Samedi"},		//fr
	{"Domenica",	"Lunedi",		"Martedi",	"Mercoledi",	"Giovedi",		"Venerdi",		"Sabato"},		//it
	{"Sonntag",		"Montag",		"Dienstag",	"Mittwoch",		"Donnerstag",	"Freitag",		"Samstag"},		//de
	{"Domingo",		"Lunes",		"Martes",	"Miércoles",	"Jueves",		"Viernes",		"Sábádo"},		//es
	{"Niedziela",	"Poniedziałek",	"Wtorek",	"Środa",		"Czwartek",		"Piątek",		"Sobota"},		//po
};

char* getDayName(enum tagLanguages lang, int day)
{
	return dayNames[lang][CHANGEBASE(day,0,0)];
}


//-----------------------------------------------------------------------------

char* monthNames[][12] = {
	{"Ιανουαρίου",	"Φεβρουαρίου",	"Μαρτίου",	"Απριλίου",	"Μαΐου",	"Ιουνίου",	"Ιουλίου",	"Αυγούστου",	"Σεπτεμβρίου",	"Οκτωβρίου",	"Νοεμβρίου",	"Δεκεμβρίου"},	//el
	{"January",		"February",		"March",	"April",	"May",		"June",		"July",		"August",		"September",	"October",		"November",		"December"},	//en
	{"Janvier",		"Février",		"Mars",		"Avril",	"Mai",		"Juin",		"Juillet",	"Août",			"Septembre",	"Octobre",		"Novembre",		"Décembre"},	//fr
	{"Gennaio",		"Febbraio",		"Marzo",	"Aprile",	"Maggio",	"Giugno",	"Luglio",	"Agosto",		"Settembre",	"Ottobre",		"Novembre",		"Dicembre"},	//it
	{"Januar",		"Februar",		"März",		"April",	"Mai",		"Juni",		"Juli",		"August",		"September",	"Oktober",		"November",		"Dezember"},	//de
	{"Enero",		"Febrero",		"Marzo",	"Abril",	"Mayo",		"Junio",	"Julio",	"Agosto",		"Septiembre",	"Octubre",		"Noviembre",	"Diciembre"},	//es
	{"Styczeń",		"Luty",			"Marzec",	"Kwiecień",	"Maj",		"Czerwiec",	"Lipiec",	"Sierpień",		"Wrzesień",		"Październik",	"Listopad",		"Grudzień"},	//po
};

char* getMonthName(enum tagLanguages lang, int month)
{
	return monthNames[lang][CHANGEBASE(month,1,0)];
}


//-----------------------------------------------------------------------------

#ifdef _WIN32
inline void EnsureUTF8(char *pStr, int len)
{
	wchar_t *UTF8Str = (wchar_t*)malloc(len * sizeof(wchar_t));

	MultiByteToWideChar( CP_ACP, 0, pStr, -1, UTF8Str, len);
	WideCharToMultiByte(CP_UTF8, 0, UTF8Str, -1, pStr, len, NULL, NULL);
	
	free(UTF8Str);
}
#else
inline void EnsureUTF8(char *pStr, int len){}
#endif

static function_entry greekutils_functions[] = {
	PHP_FE(greekutils_version, NULL)
	PHP_FE(greekutils_day, NULL)
	PHP_FE(greekutils_month, NULL)
	PHP_FE(greekutils_language, NULL)
	{NULL, NULL, NULL}
};


zend_module_entry greekutils_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
	STANDARD_MODULE_HEADER,
#endif
	PHP_GREEKUTILS_WORLD_EXTNAME,
	greekutils_functions,
	NULL,
	NULL,
	NULL,
	NULL,
	PHP_MINFO(greekutils),	//    NULL,
#if ZEND_MODULE_API_NO >= 20010901
	PHP_GREEKUTILS_WORLD_VERSION,
#endif
	STANDARD_MODULE_PROPERTIES
};


#ifdef COMPILE_DL_GREEKUTILS
ZEND_GET_MODULE(greekutils)
#endif

PHP_MINFO_FUNCTION(greekutils)
{
	php_info_print_table_start();
	php_info_print_table_row(2, "greekutils",				"Greek Utilities");
	php_info_print_table_row(2, "greekutils_language()",	"Language names");
	php_info_print_table_row(2, "greekutils_day()",			"Day names");
	php_info_print_table_row(2, "greekutils_month()",		"Month names");
	php_info_print_table_row(2, "hextocolor()",				"Convert hex to int color");
	php_info_print_table_row(2, "colortohex()",				"Conver int to hex color");
	php_info_print_table_end();
}

PHP_FUNCTION(greekutils_version)
{
    RETURN_STRING(PHP_GREEKUTILS_WORLD_VERSION, 1);
}

PHP_FUNCTION(greekutils_language)
{
	enum tagLanguages lang = el;

	if ((zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|l", &lang) == FAILURE) ||
		(lang < el) ||
		(lang >= tagLanguagesCount))
	{ RETURN_STRING("ΜΗ ΑΠΟΔΕΚΤΗ ΓΛΩΣΣΑ", 1); }

	RETURN_STRING(getLanguageName(lang), 1);
}

PHP_FUNCTION(greekutils_day)
{
	long dayNumber = 0;
	enum tagLanguages lang = el;

	if ((zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l|l", &dayNumber, &lang) == FAILURE) ||
		(dayNumber < 0) ||
		(dayNumber > 6))
	{
		RETURN_STRING("ΜΗ ΑΠΟΔΕΚΤΗ ΗΜΕΡΑ", 1);
	}
	if ((lang < el) || (lang >= tagLanguagesCount)) { RETURN_STRING("ΜΗ ΑΠΟΔΕΚΤΗ ΓΛΩΣΣΑ", 1); }

	RETURN_STRING(getDayName(lang, dayNumber), 1);
}

PHP_FUNCTION(greekutils_month)
{
	long monthNumber = 0;
	enum tagLanguages lang = el;

	if ((zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l|l", &monthNumber, &lang) == FAILURE) ||
		(monthNumber < 1) ||
		(monthNumber > 12))
	{
		RETURN_STRING("ΜΗ ΑΠΟΔΕΚΤΟΣ ΜΗΝΑΣ", 1);
	}
	if ((lang < el) || (lang >= tagLanguagesCount)) { RETURN_STRING("ΜΗ ΑΠΟΔΕΚΤΗ ΓΛΩΣΣΑ", 1); }

	RETURN_STRING(getMonthName(lang, monthNumber), 1);
}

PHP_FUNCTION(hextocolor)
{
	long color = 0;
	char szColor[] = "000000";

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &color) == FAILURE)
	{
		RETURN_STRING(szColor, 1);
	}

	Color2HEX(color, szColor);

	RETURN_STRING(szColor, 1);
}

PHP_FUNCTION(colortohex)
{
	int nsize, color = 0;
	char* str;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &str, &nsize) == FAILURE)
	{
		RETURN_LONG(0);
	}

	if (nsize > 6) str[6] = '\0';
	HEX2Color(str, &color);

	RETURN_LONG(color);
}
